#include "show.h"
#include <ctime>

using namespace std;

float k1,k2;
char lastbuf[1024];
int NAS=2;
int dnas;
dim3 d;
BITMAP *actual=NULL;
int state,n_bonds,n_horizontal_bonds;

void chyba(char *co)
{
    allegro_message(co);
    allegro_exit();
    exit(0);
}

void zaklad(dim3 dimension)
{
    dnas=dimension.x*NAS*2+10;
    actual=create_bitmap(dimension.x*NAS*2+dnas,dimension.y*NAS*2+45);
    clear_to_color(actual,makecol(255,255,255));
    rectfill(actual,0,0,dimension.x*NAS*2,dimension.z*NAS*2,makecol(32,32,32));
    rectfill(actual,dnas,0,dimension.x*NAS*2+dnas,dimension.y*NAS*2,makecol(32,32,32));
    
    textprintf_ex(actual,font,0,dimension.y*NAS*2+15,0,-1,"Dimension: %d x %d x %d",dimension.x,dimension.y,dimension.z);
    textprintf_ex(actual,font,0,dimension.y*NAS*2+25,0,-1,"Parameters: K1 = %g, K2 = %g",k1,k2);
}

void kresli(uint4 n)
{
    int x,y,z;

    x=n.x*2*NAS;
    z=n.z*2*NAS;

    if(n.w==0){
        vline(actual,x+NAS,z-NAS,z+NAS,makecol(255,255,255));
    }
    if(n.w==1){
        rectfill(actual,x-NAS+1,z-NAS+1,x+NAS-1,z+NAS-1,makecol(100,200,100));
    }
    if(n.w==2){
        hline(actual,x-NAS,z+NAS,x+NAS,makecol(255,255,255));
    }

    x=n.x*2*NAS+dnas;
    y=n.y*2*NAS;
    
    if(n.w==0){
        vline(actual,x+NAS,y-NAS,y+NAS,makecol(255,255,255));
    }
    if(n.w==1){
        hline(actual,x-NAS,y+NAS,x+NAS,makecol(255,255,255));
    }
    if(n.w==2){
        rectfill(actual,x-NAS+1,y-NAS+1,x+NAS-1,y+NAS-1,makecol(100,200,100));
    }
}

bool parse_trajectory(FILE *s)
{
    uint4 a;
    while(!feof(s)){
        if(fgets(lastbuf,sizeof(lastbuf),s)==NULL) return false;
        if(sscanf(lastbuf,"%d %d %d %d",&a.x,&a.y,&a.z,&a.w)!=4) break;
        kresli(a);
        if(key[KEY_ESC]) return false;
    }
    return true;
}

bool parse_head(FILE *s)
{
    if(fgets(lastbuf,sizeof(lastbuf),s)==NULL) return false;
    if(fscanf(s,"%f %f\n",&k1,&k2)!=2) return false;
    if(fscanf(s,"%d %d %d\n",&d.x,&d.y,&d.z)!=3) return false;
    if(fgets(lastbuf,sizeof(lastbuf),s)==NULL) return false;
    
    return true;             
}

bool parse_results(FILE *s)
{
    if(fgets(lastbuf,sizeof(lastbuf),s)==NULL) return false;
    if(sscanf(lastbuf,"%d %d %d\n",&state,&n_bonds,&n_horizontal_bonds)!=3) return false;
    if(fgets(lastbuf,sizeof(lastbuf),s)==NULL) return false;
    textprintf_ex(actual,font,0,d.y*NAS*2+35,0,-1,"RESULTS: Broken bonds = %d, Broken horizontal bonds (columns) = %d",n_bonds,n_horizontal_bonds);
    return true;             
}

bool parse_comment(FILE *s)
{
    if(fgets(lastbuf,sizeof(lastbuf),s)==NULL) return false;
    if(strlen(lastbuf)>1) lastbuf[strlen(lastbuf)-1]=0;
    textprintf_ex(actual,font,0,d.y*NAS*2+5,0,-1,"COMMENT: %s",lastbuf);
    if(fgets(lastbuf,sizeof(lastbuf),s)==NULL) return false;
    return true;             
}

void main_usage(char *name)
{
    printf("Usage: %s filename scale\n",name);
    printf("Scale should be between 1 and 5.\n");
    exit(1);
}

int main(int argc, char *argv[])
{
    if(argc<3) main_usage(argv[0]);
    
    if(sscanf(argv[2],"%d",&NAS)!=1||NAS<1||NAS>5) main_usage(argv[0]);
    
    FILE *s=fopen(argv[1],"r");
    if(s==0) main_usage(argv[0]);

    allegro_init();
    install_keyboard();
    
    set_color_depth(32);

    if(!parse_head(s)) chyba("Chybny format suboru.");
    printf("DIM: %dx%dx%d\n",d.x,d.y,d.z);
    zaklad(d);

    while(!feof(s)){
        if(!strcmp(lastbuf,"<END>\n")){
            break;
        }
        if(!strcmp(lastbuf,"<TRAJECTORY>\n")){
            if(!parse_trajectory(s)) break;
            continue;
        }
        if(!strcmp(lastbuf,"<RESULTS>\n")){
            if(!parse_results(s)) break;
            continue;
        }
        if(!strcmp(lastbuf,"<COMMENT>\n")){
            if(!parse_comment(s)) break;
            continue;
        }
        if(fgets(lastbuf,sizeof(lastbuf),s)==NULL) break;
        if(key[KEY_ESC]) break;
    }
    fclose(s);

    if(actual!=NULL){
        textprintf_ex(actual,font,2,2,makecol(255,255,255),-1,"SIDE VIEW");
        textprintf_ex(actual,font,dnas+2,2,makecol(255,255,255),-1,"TOP VIEW");
        replace_extension(lastbuf,argv[1],"bmp",sizeof(lastbuf));
        save_bitmap(lastbuf,actual,NULL);
        if(set_gfx_mode(GFX_GDI,actual->w,actual->h,0,0)==0){
            blit(actual,screen,0,0,0,0,actual->w,actual->h);
            readkey();
        }
    }
    allegro_exit();
    return 0;
}
END_OF_MAIN();
